import java.sql.SQLException;
import java.util.*;

public class TimeTrackerProc {

	class ProjectIDComparator implements Comparator {
		//A comparator for Products based on productID

		public int compare(Object arg0, Object arg1) {
			Project one = (Project) arg0;
			Project two = (Project) arg1;
			return one.projectID - two.projectID;
		}
	}

	class ProjectNameComparator implements Comparator {
		//A comparator for Products based on name

		public int compare(Object arg0, Object arg1) {
			Project one = (Project) arg0;
			Project two = (Project) arg1;
			String o = one.name;
			String t = two.name;
			int ret = o.compareToIgnoreCase(t);

			if (ret != 0)
				return ret;
			else //Settle ties with lower projectID.
				return one.projectID - two.projectID;
		}
	}

	class EmployeeIDComparator implements Comparator {
		//A comparator for Employees based on employeeID

		public int compare(Object arg0, Object arg1) {
			Employee one = (Employee) arg0;
			Employee two = (Employee) arg1;
			return one.employeeID - two.employeeID;
		}
	}

	class EmployeeNameComparator implements Comparator {
		//A comparator for Employees based on lastname, firstname

		public int compare(Object arg0, Object arg1) {
			Employee one = (Employee) arg0;
			Employee two = (Employee) arg1;
			String o = one.lastName + " " + one.firstName;
			String t = two.lastName + " " + two.firstName;
			int ret = o.compareToIgnoreCase(t);

			if (ret != 0)
				return ret;
			else //Settle ties with lower employeeID.
				return one.employeeID - two.employeeID;
		}
	}

	//Possible modes for viewing the list of employees and projects.
	public static final int VIEW_BY_ID = 0, VIEW_BY_NAME = 1;
	
	//The current active employee and project
	int activeEmployee = -1;
	int activeProject = -1;
	int activeAllProject = -1;

	//List of ALL employees, projects for selected employee and ALL projects.
	private Employee[] employeesList;
	private Project[] projectsList;
	private Project[] allProjectsList;

	//Lists of employees in a project and not in a project
	private Employee[] inProjectList;
	private Employee[] notInProjectList;

	//The current employee and project list viewing mode.
	private int curEmployeeMode = -1;
	private int curProjectMode = -1;
	private int curAllProjectMode = -1;
	
	//The employee in and not in project viewing mode.
	private int inProjectMode = -1;

	//Our database object.
	private TimeTrackerDB database;

	//Information for maintaining an "external" employeeFilter on the employee list.
	private ArrayList employeeFilter;
	private int lastEmployeeFilterMode;
	private String lastEmployeeFilterString;

	//Information for maintaining an "external" projectFilter on the project list.
	private ArrayList projectFilter;
	private int lastProjectFilterMode;
	private String lastProjectFilterString;
	
	//Information for maintaining an "external" pmProjectFilter on the all project list.
	private ArrayList allProjectFilter;
	private int lastAllProjectFilterMode;
	private String lastAllProjectFilterString;

	//List of transactions for the currently active employee.
	private Transaction[] transList;

	public TimeTrackerProc() {
		try {
			database = new TimeTrackerDB();
			employeesList = database.retrieveEmployees();
			projectsList = database.retrieveProjectsForEmployee(0);
			allProjectsList = database.retrieveProjects();
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		curEmployeeMode = VIEW_BY_NAME;
		curProjectMode = VIEW_BY_NAME;
		curAllProjectMode = VIEW_BY_NAME;
		inProjectMode = VIEW_BY_NAME;
		
		Arrays.sort(employeesList, new EmployeeNameComparator());
		
		setEmployeeFilter("");
		setProjectFilter("");
		setActiveEmployee(0);
		
		setActiveProject(0);
		
		Arrays.sort(allProjectsList, new ProjectNameComparator());
		setAllProjectFilter("");
		setActiveAllProject(0);
		updateInProjectLists();
	}

	public boolean addTransaction(
		int transID,
		int projID,
		int hrs,
		String date,
		String activity) {

		//Create a new Transaction and set the fields 
		Transaction t = new Transaction();
		t.setFields(
			transID,
			employeesList[activeEmployee].employeeID,
			projID,
			hrs,
			date,
			activity);

		//Add transaction to the database
		boolean success = database.create(t);
		updateTransList();
		return success;
	}

	public boolean updateTransaction(
		int index,
		int transID,
		int projID,
		int hrs,
		String date,
		String activity) {

		Transaction t = transList[index];

		//Keep the old ID, to be used by the database
		int oldID = t.transactionID;

		t.setFields(
			transID,
			employeesList[activeEmployee].employeeID,
			projID,
			hrs,
			date,
			activity);

		//Pass the ID of the Transaction to be updated, in case
		//the transactionID itself is modified.
		boolean success = database.update(oldID, t);
		updateTransList();
		return success;
	}

	public void deleteTransaction(int index) {
		try {
			database.delete(transList[index]);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		updateTransList();
	}

	public boolean addEmployee(
		int employeeID,
		String fName,
		String lName,
		String phone,
		String e,
		String bdate,
		String add,
		String info) {

		//Create a new Employee and set the fields
		Employee c = new Employee();
		c.setFields(employeeID, fName, lName, phone, e, bdate, add, info);

		boolean success = database.create(c);

		updateEmployeeList();
		updateInProjectLists();

		//Make the new employee the active one, if it is visible under the current employeeFilter
		for (int i = 0; i < employeeFilter.size(); i++) {
			Integer listIndex = (Integer) (employeeFilter.get(i));
			Employee c2 = employeesList[listIndex.intValue()];
			if (c2.employeeID == employeeID) {
				setActiveEmployee(i);
				break;
			}
		}

		return success;
	}

	public boolean updateEmployee(
		int employeeID,
		String fName,
		String lName,
		String phone,
		String e,
		String bdate,
		String add,
		String info) {

		int oldID = employeesList[activeEmployee].employeeID;

		Employee c = new Employee();
		c.setFields(employeeID, fName, lName, phone, e, bdate, add, info);

		boolean success = database.update(oldID, c);
		updateEmployeeList();
		updateAllProjectsList();

		//Ensure the edited employee is still the active one if it is visible under the current employeeFilter
		for (int i = 0; i < employeeFilter.size(); i++) {
			Integer listIndex = (Integer) (employeeFilter.get(i));
			Employee c2 = employeesList[listIndex.intValue()];
			if (c2.employeeID == employeeID) {
				setActiveEmployee(i);
				break;
			}
		}

		return success;
	}

	public void deleteEmployee(int index) {
		try {
			//Get the Employee from employeesList and delete the Employee
			Employee c = employeesList[((Integer) employeeFilter.get(index)).intValue()];
			database.delete(c);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		updateEmployeeList();
		setActiveEmployee(0);
		updateInProjectLists();
	}

	public boolean addProject(
		int projectID,
		String projectName,
		int managerID) {

		//Create a new Project and set its fields
		Project c = new Project();
		c.setFields(projectID, projectName, managerID);

		boolean success = database.create(c);

		if (success)
			if (!database.isEmployeeInProject(managerID, projectID))
				addEmployeeProjectByID(managerID, projectID);

		updateProjectsList();
		updateAllProjectsList();

		//Make the new project the active one, if it is visible under the current allProjectFilter
		for (int i = 0; i < allProjectFilter.size(); i++) {
			Integer listIndex = (Integer) (allProjectFilter.get(i));
			Project c2 = allProjectsList[listIndex.intValue()];
			if (c2.projectID == projectID) {
				setActiveAllProject(i);
				break;
			}
		}

		return success;
	}

	public boolean updateProject(
	int projectID,
	String projectName,
	int managerID) {

		int oldID = allProjectsList[activeAllProject].projectID;

		Project c = new Project();
		c.setFields(projectID, projectName, managerID);

		boolean success = database.update(oldID, c);

		//Add Employee to Project if the Employee is the Manager of the Project
		if (success && !database.isEmployeeInProject(managerID, projectID))
			addEmployeeProjectByID(managerID, projectID);
	
		updateProjectsList();
		updateAllProjectsList();

		//Ensure the edited project is still the active one, if it is visible under the current allProjectFilter
		for (int i = 0; i < allProjectFilter.size(); i++) {
			Integer listIndex = (Integer) (allProjectFilter.get(i));
			Project c2 = allProjectsList[listIndex.intValue()];
			if (c2.projectID == projectID) {
				setActiveAllProject(i);
				break;
			}
		}

		return success;
	}

	public void deleteProject(int index) {
		try {
			Project c = allProjectsList[((Integer) allProjectFilter.get(index)).intValue()];
			database.delete(c);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		updateProjectsList();
		updateAllProjectsList();
		setActiveAllProject(0);
	}

	public boolean addEmployeeProject(Employee employee, Project project) {
		boolean success = database.create(employee.employeeID, project.projectID);
		
		//If the Employee that was added to the Project is the active Employee,
		//update the projects list
		if (employeesList[activeEmployee].employeeID == employee.employeeID)
			updateProjectsList();
		
		updateInProjectLists();
		
		return success;
	}
	
	public boolean addEmployeeProjectByID(int employeeID, int projectID) {
		boolean success = database.create(employeeID, projectID);
		
		//If the Employee that was added to the Project is the active Employee,
		//update the projects list
		if (employeesList[activeEmployee].employeeID == employeeID)
			updateProjectsList();
		
		updateInProjectLists();
		
		return success;
	}

	public void deleteEmployeeProject(Employee employee, Project project) {
		try {
			database.delete(employee, project);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		//If the Employee that was removed from the Project is the active Employee,
		//update the projects list		
		if (employeesList[activeEmployee].employeeID == employee.employeeID)
			updateProjectsList();
		
		updateInProjectLists();
	}
	
	private void doProjectFilter(String f, int mode) {
		//Private method to carry out projectFiltering.
		//Perform projectFilter, with specified viewing mode, with projectFilter f.

		projectFilter = new ArrayList();
		
		if (mode == VIEW_BY_NAME) {
			for (int i = 0; i < projectsList.length; i++) {
				Project c = projectsList[i];
				String check = c.name.toUpperCase();

				//If the Project matches the filter, add the index of the Project to projectFilter
				if (check.indexOf(f.toUpperCase()) != -1)
					projectFilter.add(new Integer(i));
			}
		} else if (mode == VIEW_BY_ID) {
			for (int i = 0; i < projectsList.length; i++)
				//If the Project matches the filter, add the index of the Project to projectFilter
				if (("" + projectsList[i].projectID).indexOf(f) != -1)
					projectFilter.add(new Integer(i));
		} else
			System.err.println(
				"Internal Error: Unreachable code in setProjectFilter()");
	}
	
	private void doAllProjectFilter(String f, int mode) {
		//Private method to carry out allProjectFiltering.
		//Perform allProjectFilter, with specified viewing mode, with allProjectFilter f.

		allProjectFilter = new ArrayList();
		
		if (mode == VIEW_BY_NAME) {
			for (int i = 0; i < allProjectsList.length; i++) {
				Project c = allProjectsList[i];
				String check = c.name.toUpperCase();

				//If the Project matches the filter, add the index of the Project to projectFilter
				if (check.indexOf(f.toUpperCase()) != -1)
					allProjectFilter.add(new Integer(i));
			}
		} else if (mode == VIEW_BY_ID) {
			for (int i = 0; i < allProjectsList.length; i++)
				//If the Project matches the filter, add the index of the Project to projectFilter
				if (("" + allProjectsList[i].projectID).indexOf(f) != -1)
					allProjectFilter.add(new Integer(i));
		} else
			System.err.println(
				"Internal Error: Unreachable code in setAllProjectFilter()");
	}

	private void doEmployeeFilter(String f, int mode) {
		//Private method to carry out employeeFiltering.
		//Perform employeeFilter, with specified viewing mode, with employeeFilter f.

		employeeFilter = new ArrayList();
		if (mode == VIEW_BY_NAME) {
			for (int i = 0; i < employeesList.length; i++) {
				Employee c = employeesList[i];
				String check =
					c.firstName.toUpperCase() + ", " + c.lastName.toUpperCase();

				//If the Employee matches the filter, add the index of the Employee to employeeFilter
				if (check.indexOf(f.toUpperCase()) != -1)
					employeeFilter.add(new Integer(i));
			}
		} else if (mode == VIEW_BY_ID) {
			for (int i = 0; i < employeesList.length; i++)
				//If the Employee matches the filter, add the index of the Employee to employeeFilter
				if (("" + employeesList[i].employeeID).indexOf(f) != -1)
					employeeFilter.add(new Integer(i));
		} else
			System.err.println(
				"Internal Error: Unreachable code in setEmployeeFilter()");
	}

	public Project getActiveProject() {
		if (activeProject < 0 || activeProject > projectsList.length - 1)
			return null;
		return projectsList[activeProject];
	}
	
	public Project getActiveAllProject() {
		if (activeAllProject < 0 || activeAllProject > allProjectsList.length - 1)
			return null;
		return allProjectsList[activeAllProject];
	}
	
	public Project getAllProjectByIndex(int index) {
		if (index < 0 || index >= allProjectsList.length)
			return null;
		return allProjectsList[index];
	}
	
	public int getNumAllProjects() {
		return allProjectsList.length;
	}

	public Employee getActiveEmployee() {
		if (activeEmployee < 0 || activeEmployee > employeesList.length - 1)
			return null;
		return employeesList[activeEmployee];
	}

	public int getActiveProjectIndex() {
		return projectFilter.indexOf(new Integer(activeProject));
	}
	
	public int getActiveAllProjectIndex() {
		return allProjectFilter.indexOf(new Integer(activeAllProject));
	}

	public int getActiveEmployeeIndex() {
		return employeeFilter.indexOf(new Integer(activeEmployee));
	}

	public int getActiveEmployeeRealIndex() {
		return activeEmployee;
	}
	
	public Employee getEmployeeByIndex(int index) {
		if (index < 0 || index >= employeesList.length)
			return null;
		else
			return employeesList[index];	
	}

	public Employee getEmployeeInProjectByIndex(int index) {
		if (index < 0 || index >= inProjectList.length)
			return null;
		else
			return inProjectList[index];
	}
	
	public Employee getEmployeeNotInProjectByIndex(int index) {
		if (index < 0 || index >= notInProjectList.length)
			return null;
		else
			return notInProjectList[index];
	}
	
	public int getInProjectIndex(int employeeID) {
		//Return the index of an Employee in inProjectList based on the Employee ID
		
		int index = -1;
		
		for (int i = 0; i < inProjectList.length; i++) {
			if (inProjectList[i].employeeID == employeeID) {
				index = i;
				break;
			}
		}
		
		return index;
	}

	public int getNotInProjectIndex(int employeeID) {
		//Return the index of an Employee in notInProjectList based on the Employee ID

		int index = -1;
		
		for (int i = 0; i < notInProjectList.length; i++) {
			if (notInProjectList[i].employeeID == employeeID) {
				index = i;
				break;
			}
		}
		
		return index;
	}

	public String[] getProjectList() {
		//Create and return a list of Strings representing
		//the available projects for the current projectFilter mode.

		String[] list = new String[projectFilter.size()];
		for (int i = 0; i < projectFilter.size(); i++) {
			int realIndex = ((Integer) projectFilter.get(i)).intValue();
			if (curProjectMode == VIEW_BY_NAME)
				list[i] = projectsList[realIndex].name;
			else if (curProjectMode == VIEW_BY_ID)
				list[i] = "" + projectsList[realIndex].projectID;
		}
		return list;
	}
	
	public String[] getAllProjectList() {
		//Create and return a list of Strings representing
		//the available projects for the current allProjectFilter mode.

		String[] list = new String[allProjectFilter.size()];
		for (int i = 0; i < allProjectFilter.size(); i++) {
			int realIndex = ((Integer) allProjectFilter.get(i)).intValue();
			if (curAllProjectMode == VIEW_BY_NAME)
				list[i] = allProjectsList[realIndex].name;
			else if (curAllProjectMode == VIEW_BY_ID)
				list[i] = "" + allProjectsList[realIndex].projectID;
		}
		return list;
	}

	public String[] getEmployeeList() {
		//Create and return a list of Strings representing
		//the available employees for the current employeeFilter mode.

		String[] list = new String[employeeFilter.size()];
		for (int i = 0; i < employeeFilter.size(); i++) {
			int realIndex = ((Integer) employeeFilter.get(i)).intValue();
			if (curEmployeeMode == VIEW_BY_NAME)
				list[i] =
					employeesList[realIndex].lastName
						+ ", "
						+ employeesList[realIndex].firstName;
			else if (curEmployeeMode == VIEW_BY_ID)
				list[i] = "" + employeesList[realIndex].employeeID;
		}
		return list;
	}

	public String[] getEmployeeListAll() {
		//Create and return a list of Strings representing all Employees

		String[] list = new String[employeesList.length];
		for (int i = 0; i < employeesList.length; i++)
			list[i] = employeesList[i].lastName + ", " + employeesList[i].firstName;
		return list;
	}
	
	public String[] getInProjectList() {
		//Create and return a list of Strings representing all Employees in activeAllProject

		String[] list = new String[inProjectList.length];
		for (int i = 0; i < inProjectList.length; i++) {
			if (inProjectMode == VIEW_BY_NAME)
				list[i] = inProjectList[i].lastName	+ ", " + inProjectList[i].firstName;
			else if (inProjectMode == VIEW_BY_ID)
				list[i] = "" + inProjectList[i].employeeID;
		}
		return list;
	}
	
	public String[] getNotInProjectList() {
		//Create and return a list of Strings representing all Employees not in activeAllProject

		String[] list = new String[notInProjectList.length];
		for (int i = 0; i < notInProjectList.length; i++) {
			if (inProjectMode == VIEW_BY_NAME)
				list[i] = notInProjectList[i].lastName + ", "+ notInProjectList[i].firstName;
			else if (inProjectMode == VIEW_BY_ID)
				list[i] = "" + notInProjectList[i].employeeID;
		}
		return list;
	}
	
	public int getEmployeeRealIndexByID(int employeeID) {
		//Returns the index of an Employee in emploeesList by the Employee ID
		
		int index = -1;
		
		for (int i = 0; i < employeesList.length; i++) {
			if (employeesList[i].employeeID == employeeID) {
				index = i;
				break;
			}
		}
		
		return index;
	}

	public int getNumTransactions() {
		return transList.length;
	}

	public Transaction getTransactionByID(int transID) {
		for (int i = 0; i < transList.length; i++) {
			if (transList[i].transactionID == transID)
				return transList[i];
		}
		return null;
	}

	public Transaction getTransactionByIndex(int i) {
		if (i < 0 || i >= transList.length)
			return null;

		Transaction t = transList[i];
		return t;
	}

	public int getTransTotal() {
		int total = 0;
		for (int i = 0; i < transList.length; i++)
			total += transList[i].hours;
		return total;
	}
	
	public Transaction[] getTransactionsByEmployee(int employeeID) {
		try {
			return database.retrieveTransactions(employeeID);
		} catch(SQLException e) {
			e.printStackTrace();
		}
		return new Transaction[0];
	}

	public void setActiveProject(int index) {
		//Ignore incorrect indexes
		if (index < 0 || index >= projectFilter.size())
			return;

		//Set the activeProject index into projectsList.
		//ie. translate projectFilter (external) index into "real" index.
		if (projectsList.length > 0)
			activeProject = ((Integer) projectFilter.get(index)).intValue();
		else
			activeProject = -1;

		//Refresh the transaction list after changing the active Project.
		updateTransList();
	}

	public void setActiveAllProject(int index) {
		//Ignore incorrect indexes
		if (index < 0 || index >= allProjectFilter.size())
			return;

		//Set the activeAllProject index into allProjectsList.
		//ie. translate allProjectFilter (external) index into "real" index.
		if (allProjectsList.length > 0)
			activeAllProject = ((Integer) allProjectFilter.get(index)).intValue();
		else
			activeAllProject = -1;
		
		updateInProjectLists();
	}

	public void setActiveEmployee(int index) {
		//Ignore incorrect indexes
		if (index < 0 || index >= employeeFilter.size())
			return;

		//Set the activeEmployee index into employeesList.
		//ie. translate employeeFilter (external) index into "real" index.
		activeEmployee = ((Integer) employeeFilter.get(index)).intValue();

		//Refresh the transaction list after changing the active Employee.
		updateProjectsList();
	}

	public void setProjectView(int newMode) {
		//Ignore modes that don't exist.
		if (newMode != VIEW_BY_ID && newMode != VIEW_BY_NAME)
			return;

		if (curProjectMode != newMode) {
			curProjectMode = newMode;
			sortProjects();
		}
	}
	
	public void setAllProjectView(int newMode) {
		//Ignore modes that don't exist.
		if (newMode != VIEW_BY_ID && newMode != VIEW_BY_NAME)
			return;

		if (curAllProjectMode != newMode) {
			curAllProjectMode = newMode;
			sortAllProjects();
		}
	}

	public void setEmployeeView(int newMode) {
		//Ignore modes that don't exist.
		if (newMode != VIEW_BY_ID && newMode != VIEW_BY_NAME)
			return;

		if (curEmployeeMode != newMode) {
			curEmployeeMode = newMode;
			sortEmployees();
		}
	}
	
	public void setInProjectView(int newMode) {
		//Ignore modes that don't exist.
		if (newMode != VIEW_BY_ID && newMode != VIEW_BY_NAME)
			return;

		if (inProjectMode != newMode) {
			inProjectMode = newMode;
			sortInProjectLists();
		}
	}

	private void sortProjects() {
		if (activeProject == -1)
			return;
		
		//Store the ID of the active project, so
		//it can be maintained after our sorting.
		int activeProjectID = projectsList[activeProject].projectID;

		//Sort the project list using the new viewing mode.
		if (curProjectMode == VIEW_BY_ID)
			Arrays.sort(projectsList, new ProjectIDComparator());
		else if (curProjectMode == VIEW_BY_NAME)
			Arrays.sort(projectsList, new ProjectNameComparator());
		else
			System.err.println("Error: unreachable code in setProjectView");

		//Perform the same projectFilter that was already active.
		doProjectFilter(lastProjectFilterString, lastProjectFilterMode);

		//Restore the index of the activeProject to its new value.
		for (int i = 0; i < projectsList.length; i++) {
			if (projectsList[i].projectID == activeProjectID) {
				activeProject = i;
				break;
			}
		}
	}

	private void sortAllProjects() {
		if (activeAllProject == -1) {
			doAllProjectFilter(lastAllProjectFilterString, lastAllProjectFilterMode);
			return;
		}
		
		//Store the ID of the active project, so
		//it can be maintained after our sorting.
		int activeProjectID = allProjectsList[activeAllProject].projectID;

		//Sort the project list using the new viewing mode.
		if (curAllProjectMode == VIEW_BY_ID)
			Arrays.sort(allProjectsList, new ProjectIDComparator());
		else if (curAllProjectMode == VIEW_BY_NAME)
			Arrays.sort(allProjectsList, new ProjectNameComparator());
		else
			System.err.println("Error: unreachable code in setProjectView");

		//Perform the same projectFilter that was already active.
		doAllProjectFilter(lastAllProjectFilterString, lastAllProjectFilterMode);

		//Restore the index of the activeAllProject to its new value.
		for (int i = 0; i < allProjectsList.length; i++) {
			if (allProjectsList[i].projectID == activeProjectID) {
				activeAllProject = i;
				break;
			}
		}
	}
	
	private void sortInProjectLists() {
		if (activeAllProject == -1)
			return;

		//Sort the list using the new viewing mode.
		if (inProjectMode == VIEW_BY_ID) {
			Arrays.sort(inProjectList, new EmployeeIDComparator());
			Arrays.sort(notInProjectList, new EmployeeIDComparator());
		}
		else if (inProjectMode == VIEW_BY_NAME) {
			Arrays.sort(inProjectList, new EmployeeNameComparator());
			Arrays.sort(notInProjectList, new EmployeeNameComparator());
		}
		else
			System.err.println("Error: unreachable code in setInProjectView");
	}

	private void sortEmployees() {
		if (activeEmployee == -1) {
			//Perform the same employeeFilter that was already active.
			doEmployeeFilter(lastEmployeeFilterString, lastEmployeeFilterMode);
			return;
		}
		
		//Store the ID of the active employee, so
		//it can be maintained after our sorting.
		int activeEmployeeID = employeesList[activeEmployee].employeeID;

		//Sort the employee list using the new viewing mode.
		if (curEmployeeMode == VIEW_BY_ID)
			Arrays.sort(employeesList, new EmployeeIDComparator());
		else if (curEmployeeMode == VIEW_BY_NAME)
			Arrays.sort(employeesList, new EmployeeNameComparator());
		else
			System.err.println("Error: unreachable code in setEmployeeView");

		//Perform the same employeeFilter that was already active.
		doEmployeeFilter(lastEmployeeFilterString, lastEmployeeFilterMode);

		//Restore the index of the activeEmployee to its new value.
		for (int i = 0; i < employeesList.length; i++) {
			if (employeesList[i].employeeID == activeEmployeeID) {
				activeEmployee = i;
				break;
			}
		}
	}

	public void setProjectFilter(String fstring) {
		//Store the projects's requested projectFilter and current view mode.
		//This is used to restore the projectFilter when we switch views back and forth.
		lastProjectFilterMode = curProjectMode;
		lastProjectFilterString = fstring;

		doProjectFilter(fstring, curProjectMode);
	}
	
	public void setAllProjectFilter(String fstring) {
		//Store the projects's requested allProjectFilter and current view mode.
		//This is used to restore the allProjectFilter when we switch views back and forth.
		lastAllProjectFilterMode = curAllProjectMode;
		lastAllProjectFilterString = fstring;

		doAllProjectFilter(fstring, curAllProjectMode);
	}

	public void setEmployeeFilter(String fstring) {
		//Store the employee's requested employeeFilter and current view mode.
		//This is used to restore the employeeFilter when we switch views back and forth.
		lastEmployeeFilterMode = curEmployeeMode;
		lastEmployeeFilterString = fstring;

		doEmployeeFilter(fstring, curEmployeeMode);
	}

	private void updateTransList() {
		if (activeEmployee == -1 || activeProject == -1)
			return;
		
		try {
			transList =
				database.retrieveTransactions(
					employeesList[activeEmployee].employeeID,
					projectsList[activeProject].projectID);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	private void updateProjectsList() {
		if (activeEmployee == -1)
			return;		
		
		try {
			projectsList =
				database.retrieveProjectsForEmployee(
					employeesList[activeEmployee].employeeID);
			
			if (projectsList.length > 0)
				activeProject = 0;
			else
				activeProject = -1;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		sortProjects();
	}

	private void updateAllProjectsList() {
		try {
			allProjectsList = database.retrieveProjects();
			
			if (allProjectsList.length > 0)
				activeAllProject = 0;
			else
				activeAllProject = -1;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		sortAllProjects();
		
		if (activeAllProject != -1)
			activeAllProject = 0;
			
		updateInProjectLists();
	}

	private void updateInProjectLists() {
		if (activeAllProject == -1) {
			inProjectList = new Employee[0];
			notInProjectList = new Employee[0];
			return;
		}
		
		try {
			inProjectList = 
				database.retrieveEmployeesInProject(allProjectsList[activeAllProject].projectID);
			notInProjectList = 
				database.retrieveEmployeesNotInProject(allProjectsList[activeAllProject].projectID);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		sortInProjectLists();
	}
	
	private void updateEmployeeList() {
		try {
			employeesList = database.retrieveEmployees();
		} catch (SQLException e) {
			e.printStackTrace();
		}

		if (employeesList.length > 0)
			activeEmployee = 0;
		else
			activeEmployee = -1;
			
		sortEmployees();
	}

	public void close() {
		try {
			database.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
