import java.text.SimpleDateFormat;
import java.util.Arrays;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.widgets.*;

public class TimeTrackerUI {

	//@Version ("0.2.3")
	private static final String VERSION = "V0.2.3";

	private static final int VIEW_MODE = 0;
	private static final int ADD_MODE = 1;
	private static final int EDIT_MODE = 2;

	//Sorting criterion for transactions table
	private static final int SORT_BY_ID = 0;
	private static final int SORT_BY_AMT = 1;
	private static final int SORT_BY_DATE = 2;
	private static final int SORT_BY_DESC = 3;

	//Size constants
	//@Version ("0.2.2") 600x480 --> 800x600
	private static final int SHELL_HEIGHT = 600;
	private static final int SHELL_WIDTH = 800;
	
	private Display display;
	private Shell shell;

	private List employeeList, projectList, pmProjectList;
	private Text employeeFilterText;
	private Text projectFilterText;
	private Text pmProjectFilterText;

	private Combo currentEmployeeCombo;

	private TabFolder folder;
	private TabItem administrationItem, employeeItem, projectManagerItem;
	private Composite administrationPane, employeePane, projectManagerPane;
	private Group employeeListComp, projectListComp, currentEmployeeComp;
	private Group administrationInfo, employeeInfo;

	//Employee page widgets
	private Text acctIDText;
	private Text addressText;
	private Text fnameText;
	private Text snameText;
	private Text dobText;
	private Text phoneText;
	private Text miscInfoText;
	private Text emailText;

	//Employee modification buttons
	private Button newEmployeeButton;
	private Button editEmployeeButton;
	private Button delEmployeeButton;
	private Button saveEmployeeButton;
	private Button cancelEmployeeButton;
	private Button printTransButton;
	private StyledText printText;

	private Button employeeNameRadio;
	private Button employeeNumRadio;

	//Project Manager page widgets
	private Text projectIDText;
	private Text projectNameText;
	private Combo pmCombo;
	
	//Project Manager modification buttons
	private Button newProjectButton;
	private Button delProjectButton;
	private Button editProjectButton;
	private Button saveProjectButton;
	private Button cancelProjectButton;	
	
	private Button assignButton;
	private Button unassignButton;
	
	private Button pmProjectNumRadio;
	private Button pmProjectNameRadio;
	
	//Project Manager Assign Lists and radio buttons
	private List notInProjectList;
	private List inProjectList;
	private Button pmEmployeeNameRadio;
	private Button pmEmployeeNumRadio;

	//Transaction page widgets
	private Table infoTable;
	private Text idText;
	private Text amtText;
	private Text dateText;
	private Text descText;
	private Text totalText;

	//Transaction modification buttons
	private Button newTransButton;
	private Button editTransButton;
	private Button delTransButton;
	private Button saveTransButton;
	private Button cancelTransButton;

	private Button projectNameRadio;
	private Button projectNumRadio;

	//Processor, acts as middle-man between database class and UI
	private TimeTrackerProc processor;

	//Index of the transaction being edited. -1 if none.
	private int editIndex = -1;

	//Current mode
	private int transactionMode = VIEW_MODE;
	private int employeeMode = VIEW_MODE;
	private int projectMode = VIEW_MODE;

	private TableColumn numberColumn;
	private TableColumn amountColumn;
	private TableColumn dateColumn;
	private TableColumn descColumn;

	TimeTrackerUI() {
		try {
			processor = new TimeTrackerProc();
		}
		catch (Exception e) {
			System.err.println("Couldn't open a connection to the database.");
		}

		processor.setEmployeeView(TimeTrackerProc.VIEW_BY_NAME);
		processor.setProjectView(TimeTrackerProc.VIEW_BY_NAME);
		processor.setAllProjectView(TimeTrackerProc.VIEW_BY_NAME);
		processor.setInProjectView(TimeTrackerProc.VIEW_BY_NAME);

		display = new Display();
		shell = new Shell(display);
		shell.setSize(SHELL_WIDTH, SHELL_HEIGHT);
		shell.setText("Time Tracker " + VERSION);
		Image image1 = new Image(null, "splash.jpg");
		shell.setImage(image1);
		shell.setLayout(new FormLayout());

		setupFolders();
		setupFileMenu();
	}

	public void run() {
		refreshEmployeeData();
		refreshPMProjectData();
		refreshAssignData();
		shell.open();
		while (!shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		//Terminate the database connection
		processor.close();

		display.dispose();
	}

	private void setupFolders() {

		folder = new TabFolder(shell, SWT.NONE);
		FormLayout folderLayout = new FormLayout();
		folderLayout.marginHeight = 0;
		folderLayout.marginWidth = 0;
		folder.setLayout(folderLayout);
		FormData data = new FormData();
		data.top = new FormAttachment(0, 0);
		data.right = new FormAttachment(100, 0);
		data.left = new FormAttachment(0, 0);
		data.bottom = new FormAttachment(100, 0);
		folder.setLayoutData(data);

		printText = new StyledText(shell, SWT.BORDER);
		printText.setVisible(false);

		setupEmployeePane();

		setupAdministrationPane();

		setupProjectManagerPane();
	}

	private void clearTransactionData() {

		employeeInfo.setText("Transactions for Project:");

		//Clear Transaction table
		infoTable.removeAll();

		//Reset the text fields on the transaction page to blank.
		if (transactionMode == VIEW_MODE) {
			idText.setText("");
			amtText.setText("");
			dateText.setText("");
			descText.setText("");
		}
	}

	private void refreshTransactionData() {

		//if a project is currectly selected, display it's transactions
		if (projectList.getSelectionIndex() >= 0) {
			processor.setActiveProject(projectList.getSelectionIndex());

			Employee c = processor.getActiveEmployee();
			Project p = processor.getActiveProject();

			employeeInfo.setText("Transactions for Project: " + p.name);

			//Refill transaction table with new employee.
			infoTable.removeAll();
			int max = processor.getNumTransactions();
			
			for (int i = 0; i < max; i++) {
				Transaction current = processor.getTransactionByIndex(i);
				TableItem item = new TableItem(infoTable, SWT.NONE);

				//Remove line breaks from description, and replaces with spaces.
				StringBuffer descCopy = new StringBuffer(current.activity);
				int position;
				while ((position = descCopy.indexOf("\n")) != -1)
					descCopy.replace(position, position + 1, " ");
				while ((position = descCopy.indexOf("\r")) != -1)
					descCopy.deleteCharAt(position);

				item.setText(
					new String[] {
						"" + current.transactionID,
						"" + current.hours,
						current.date,
						descCopy.toString()});
			}

			//Reset the text fields on the transaction page to blank.
			if (transactionMode == VIEW_MODE) {
				idText.setText("");
				amtText.setText("");
				dateText.setText("");
				descText.setText("");
			}

			//Set the total text field.
			int total = processor.getTransTotal();
			totalText.setText(""+total);
		}
		else {
			clearTransactionData();
		}
	}

	private void refreshProjectData() {

		//Load data from processor and fill the list of projects.
		int selected = processor.getActiveProjectIndex();
		String[] data = processor.getProjectList();
		projectList.removeAll();
		for (int i = 0; i < data.length; i++)
			projectList.add(data[i]);

		if (selected == -1)
			selected = 0;

		projectList.setSelection(selected);

		refreshTransactionData();
	}

	private void refreshPMProjectData() {
		//Load data from processor and fill the list of projects.
		int selected = processor.getActiveAllProjectIndex();
		String[] data = processor.getAllProjectList();
		pmProjectList.removeAll();
		for (int i = 0; i < data.length; i++)
			pmProjectList.add(data[i]);

		//If there are no projects in the list
		if (data.length <= 0) {
			projectIDText.setText("");
			projectNameText.setText("");
			pmCombo.removeAll();
			inProjectList.removeAll();
			notInProjectList.removeAll();
			return;
		}
		
		//If there are projects in the list but none is selected
		else if (selected == -1) {
			selected = 0;
			processor.setActiveAllProject(0);
		}

		pmProjectList.setSelection(selected);
		
		Project c = processor.getActiveAllProject();
		
		//If there is no active project
		if (c == null)
			return;
		
		projectIDText.setText("" + c.projectID);
		projectNameText.setText(c.name);
		
		//Add all the employees to the Project Manager combo
		pmCombo.removeAll();

		String[] alldata = processor.getEmployeeListAll();
		
		for (int i = 0; i < alldata.length; i++)
			pmCombo.add(alldata[i]);

		pmCombo.select(processor.getEmployeeRealIndexByID(c.managerID));
	}

	private void refreshAssignData() {
		inProjectList.removeAll();
		notInProjectList.removeAll();
		
		//Load the inProject list
		String[] data = processor.getInProjectList();
		for (int i = 0; i < data.length; i++)
			inProjectList.add(data[i]);
			
		//Load the notInProject list
		data = processor.getNotInProjectList();
		for (int i = 0; i < data.length; i++)
			notInProjectList.add(data[i]);
		
		if (notInProjectList.getItemCount() > 0)
			notInProjectList.select(0);
		else
			inProjectList.select(0);
	}

	private void refreshEmployeeData() {

		//Load data from processor and fill the list of employees.
		int selected = processor.getActiveEmployeeIndex();
		String[] data = processor.getEmployeeList();

		//Load the Employee list
		employeeList.removeAll();

		for (int i = 0; i < data.length; i++)
			employeeList.add(data[i]);

		if (selected == -1)
			selected = 0;

		employeeList.setSelection(selected);

		processor.setActiveEmployee(selected);

		//Update the Employee Combo
		int realSelected = processor.getActiveEmployeeRealIndex();
		String[] alldata = processor.getEmployeeListAll();

		currentEmployeeCombo.removeAll();
		
		for (int i = 0; i < alldata.length; i++)
			currentEmployeeCombo.add(alldata[i]);
		
		currentEmployeeCombo.select(realSelected);

		Employee c = processor.getActiveEmployee();

		//If there is no active employee
		if (c == null) {
			//Set the label for the active employee.
			administrationInfo.setText("Employee Information:");
			
			//Set the values of the textfields for the employee page.
			acctIDText.setText("");
			fnameText.setText("");
			snameText.setText("");
			phoneText.setText("");
			emailText.setText("");
			dobText.setText("");
			addressText.setText("");
			miscInfoText.setText("");
		}
		
		//If there is an active employee.
		else {
			//Set the label for the active employee.
			administrationInfo.setText("Employee: " + c);

			//Set the values of the textfields for the employee page.
			acctIDText.setText("" + c.employeeID);
			fnameText.setText(c.firstName);
			snameText.setText(c.lastName);
			phoneText.setText(c.phone);
			emailText.setText(c.email);
			dobText.setText(c.dob);
			addressText.setText(c.address);
			miscInfoText.setText(c.miscInfo);
		}

		//Update the projects for the employee
		processor.setProjectFilter(projectFilterText.getText());
		processor.setActiveProject(0);
		refreshProjectData();
	}

	private void setEmployeeMode(int mode) {
		//Set the mode for the employee pane. (Viewing, editing, or adding).
		if (employeeMode == mode)
			return;

		//Make the modes exclusive, ie. can't edit transactions and employees simultaneously
		setTransMode(VIEW_MODE);
		setProjectMode(VIEW_MODE);

		if (mode == ADD_MODE) {
			//Empty the text fields
			administrationInfo.setText("Employee: New Employee");

			employeeList.deselectAll();

			acctIDText.setText("");
			acctIDText.setEditable(true);
			addressText.setText("");
			addressText.setEditable(true);
			fnameText.setText("");
			fnameText.setEditable(true);
			snameText.setText("");
			snameText.setEditable(true);
			dobText.setText("YYYY-MM-DD");
			dobText.setEditable(true);
			phoneText.setText("");
			phoneText.setEditable(true);
			miscInfoText.setText("");
			miscInfoText.setEditable(true);
			emailText.setText("");
			emailText.setEditable(true);

			newEmployeeButton.setEnabled(false);
			delEmployeeButton.setEnabled(false);
			editEmployeeButton.setEnabled(false);

			saveEmployeeButton.setEnabled(true);
			cancelEmployeeButton.setEnabled(true);

			printTransButton.setEnabled(false);

			employeeList.setEnabled(false);
			employeeFilterText.setEnabled(false);
			employeeNameRadio.setEnabled(false);
			employeeNumRadio.setEnabled(false);

			//Move the cursor to the ID field for convenience
			acctIDText.setFocus();
		}
		else if (mode == EDIT_MODE) {
			//refuse edit mode, if no entry is selected

			acctIDText.setEditable(true);
			addressText.setEditable(true);
			fnameText.setEditable(true);
			snameText.setEditable(true);
			dobText.setEditable(true);
			phoneText.setEditable(true);
			miscInfoText.setEditable(true);
			emailText.setEditable(true);

			newEmployeeButton.setEnabled(false);
			delEmployeeButton.setEnabled(false);
			editEmployeeButton.setEnabled(false);

			saveEmployeeButton.setEnabled(true);
			cancelEmployeeButton.setEnabled(true);

			printTransButton.setEnabled(false);

			employeeList.setEnabled(false);
			employeeFilterText.setEnabled(false);
			employeeNameRadio.setEnabled(false);
			employeeNumRadio.setEnabled(false);

			//move the cursor to the ID field
			acctIDText.setFocus();
		}
		else if (mode == VIEW_MODE) {

			acctIDText.setEditable(false);
			addressText.setEditable(false);
			fnameText.setEditable(false);
			snameText.setEditable(false);
			dobText.setEditable(false);
			phoneText.setEditable(false);
			miscInfoText.setEditable(false);
			emailText.setEditable(false);

			newEmployeeButton.setEnabled(true);
			delEmployeeButton.setEnabled(true);
			editEmployeeButton.setEnabled(true);

			saveEmployeeButton.setEnabled(false);
			cancelEmployeeButton.setEnabled(false);

			printTransButton.setEnabled(true);

			employeeList.setEnabled(true);
			employeeFilterText.setEnabled(true);
			employeeNameRadio.setEnabled(true);
			employeeNumRadio.setEnabled(true);
		}

		employeeMode = mode;
	}

	private void setProjectMode(int mode) {
		//Set the mode for the project manager pane. (Viewing, editing, or adding).
		if (projectMode == mode)
			return;

		//Make the modes exclusive, ie. can't edit transactions and employees simultaneously
		setTransMode(VIEW_MODE);
		setEmployeeMode(VIEW_MODE);

		if (mode == ADD_MODE) {
			//Empty the text fields
			projectIDText.setText("");
			projectIDText.setEditable(true);
			projectNameText.setText("");
			projectNameText.setEditable(true);

			//Load project manager combo
			pmCombo.removeAll();

			//Add all the employees to the Project Manager combo
			String[] alldata = processor.getEmployeeListAll();
		
			for (int i = 0; i < alldata.length; i++)
				pmCombo.add(alldata[i]);

			pmCombo.select(0);
			pmCombo.setEnabled(true);

			newProjectButton.setEnabled(false);
			delProjectButton.setEnabled(false);
			editProjectButton.setEnabled(false);

			saveProjectButton.setEnabled(true);
			cancelProjectButton.setEnabled(true);
			
			notInProjectList.setEnabled(false);
			inProjectList.setEnabled(false);
			assignButton.setEnabled(false);
			unassignButton.setEnabled(false);
			pmEmployeeNameRadio.setEnabled(false);
			pmEmployeeNumRadio.setEnabled(false);
			
			pmProjectList.setEnabled(false);
			pmProjectFilterText.setEnabled(false);
			pmProjectNameRadio.setEnabled(false);
			pmProjectNumRadio.setEnabled(false);

			//Move the cursor to the ID field for convenience
			projectIDText.setFocus();
		}
		else if (mode == EDIT_MODE) {
			//refuse edit mode, if no entry is selected
			if (pmProjectList.getSelectionIndex() == -1) {
				MessageBox msgBox =
					new MessageBox(
						shell,
						SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("No Selection");
				msgBox.setMessage("Please select a project to edit.");
				msgBox.open();

				return;
			}

			projectIDText.setEditable(true);
			projectNameText.setEditable(true);
			
			pmCombo.setEnabled(true);

			newProjectButton.setEnabled(false);
			delProjectButton.setEnabled(false);
			editProjectButton.setEnabled(false);

			saveProjectButton.setEnabled(true);
			cancelProjectButton.setEnabled(true);
			
			notInProjectList.setEnabled(false);
			inProjectList.setEnabled(false);
			assignButton.setEnabled(false);
			unassignButton.setEnabled(false);
			pmEmployeeNameRadio.setEnabled(false);
			pmEmployeeNumRadio.setEnabled(false);
			
			pmProjectList.setEnabled(false);
			pmProjectFilterText.setEnabled(false);
			pmProjectNameRadio.setEnabled(false);
			pmProjectNumRadio.setEnabled(false);

			//move the cursor to the ID field
			projectIDText.setFocus();
		}
		else if (mode == VIEW_MODE) {

			projectIDText.setEditable(false);
			projectNameText.setEditable(false);

			pmCombo.setEnabled(false);

			newProjectButton.setEnabled(true);
			delProjectButton.setEnabled(true);
			editProjectButton.setEnabled(true);

			saveProjectButton.setEnabled(false);
			cancelProjectButton.setEnabled(false);
			
			notInProjectList.setEnabled(true);
			inProjectList.setEnabled(true);
			assignButton.setEnabled(true);
			unassignButton.setEnabled(true);
			pmEmployeeNameRadio.setEnabled(true);
			pmEmployeeNumRadio.setEnabled(true);
			
			pmProjectList.setEnabled(true);
			pmProjectFilterText.setEnabled(true);
			pmProjectNameRadio.setEnabled(true);
			pmProjectNumRadio.setEnabled(true);
			
			refreshPMProjectData();
		}

		projectMode = mode;
	}

	private void setTransMode(int mode) {
		//Set the mode for the transaction pane. (Viewing, editing, or adding).
		if (transactionMode == mode)
			return;

		//Make the modes exclusive, ie. can't edit transactions and employees simultaneously
		setEmployeeMode(VIEW_MODE);
		setProjectMode(VIEW_MODE);

		if (mode == ADD_MODE) {
			//refuse add mode if no project is selected
			if (projectList.getSelectionIndex() == -1) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"In order to add a transaction, a project must be selected.");
				msgBox.open();
				return;
			}
			//Empty the text fields
			idText.setText("");
			idText.setEditable(true);
			amtText.setText("");
			amtText.setEditable(true);
			descText.setText("");
			descText.setEditable(true);

			//Set the date field to the current date
			java.util.Date currentDate = new java.util.Date();
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			String dateString = sdf.format(currentDate);
			dateText.setText(dateString);
			dateText.setEditable(true);

			newTransButton.setEnabled(false);
			delTransButton.setEnabled(false);
			editTransButton.setEnabled(false);

			saveTransButton.setEnabled(true);
			cancelTransButton.setEnabled(true);

			infoTable.setEnabled(false);

			currentEmployeeCombo.setEnabled(false);
			projectList.setEnabled(false);
			projectFilterText.setEnabled(false);
			projectNameRadio.setEnabled(false);
			projectNumRadio.setEnabled(false);

			//Move the cursor to the ID field for convenience
			idText.setFocus();
		}
		else if (mode == EDIT_MODE) {
			//refuse edit mode, if no entry is selected
			if (infoTable.getSelectionIndex() < 0) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"In order to edit a transaction, one must be selected.");
				msgBox.open();
				return;
			}

			idText.setEditable(true);
			amtText.setEditable(true);
			descText.setEditable(true);
			dateText.setEditable(true);

			newTransButton.setEnabled(false);
			delTransButton.setEnabled(false);
			editTransButton.setEnabled(false);

			saveTransButton.setEnabled(true);
			cancelTransButton.setEnabled(true);

			infoTable.setEnabled(false);

			currentEmployeeCombo.setEnabled(false);
			projectList.setEnabled(false);
			projectFilterText.setEnabled(false);
			projectNameRadio.setEnabled(false);
			projectNumRadio.setEnabled(false);

			//move the cursor to the ID field
			idText.setFocus();
		}
		else if (mode == VIEW_MODE) {

			int index;
			if ((index = infoTable.getSelectionIndex()) != -1) {

				int id = Integer.parseInt(infoTable.getItem(index).getText(0));

				Transaction current = processor.getTransactionByID(id);
				idText.setText("" + current.transactionID);
				dateText.setText(current.date);
				descText.setText(current.activity);
				amtText.setText("" + current.hours);
			}
			else {
				idText.setText("");
				amtText.setText("");
				dateText.setText("");
				descText.setText("");
			}

			idText.setEditable(false);
			amtText.setEditable(false);
			descText.setEditable(false);
			dateText.setEditable(false);

			//update the transaction total
			int total = processor.getTransTotal();
			totalText.setText(""+total);

			newTransButton.setEnabled(true);
			delTransButton.setEnabled(true);
			editTransButton.setEnabled(true);

			saveTransButton.setEnabled(false);
			cancelTransButton.setEnabled(false);

			infoTable.setEnabled(true);

			currentEmployeeCombo.setEnabled(true);
			projectList.setEnabled(true);
			projectFilterText.setEnabled(true);
			projectNameRadio.setEnabled(true);
			projectNumRadio.setEnabled(true);
		}
		transactionMode = mode;
	}

	private void setupEmployeeList() {
		//Create employee list.
		employeeListComp = new Group(administrationPane, SWT.NONE);
		employeeListComp.setText("Employees:");

		//Form layout data for composite
		FormData formData = new FormData();
		formData.top = new FormAttachment(0, 5);
		formData.left = new FormAttachment(0, 5);
		formData.bottom = new FormAttachment(100, -5);
		formData.right = new FormAttachment(33, -5);
		employeeListComp.setLayoutData(formData);

		//Composite has a grid layout
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		employeeListComp.setLayout(gridLayout);

		employeeList =
			new List(
				employeeListComp,
				SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 2;
		employeeList.setLayoutData(data);

		employeeList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(VIEW_MODE);
				setEmployeeMode(VIEW_MODE);
				processor.setActiveEmployee(employeeList.getSelectionIndex());
				refreshEmployeeData();
			}
		});

		//Create the filter text field and its listeners.
		Label filterLabel = new Label(employeeListComp, SWT.NONE);
		filterLabel.setText("Filter:");
		employeeFilterText = new Text(employeeListComp, SWT.BORDER);
		GridData data1 = new GridData();
		//@Version ("0.2.2") - Changed from 26 to 34
		data1.widthHint = 34;
		filterLabel.setLayoutData(data1);
		GridData data2 = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		data2.widthHint = 48;
		employeeFilterText.setLayoutData(data2);

		employeeFilterText.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent arg0) {
				processor.setEmployeeFilter(employeeFilterText.getText());
				refreshEmployeeData();
			}
		});

		employeeFilterText.addListener(SWT.DefaultSelection, new Listener() {
			public void handleEvent(Event e) { //check for "enter" key
				processor.setEmployeeFilter(employeeFilterText.getText());
				refreshEmployeeData();
			}
		});

		//Create radio buttons to switch viewing mode.
		employeeNameRadio = new Button(employeeListComp, SWT.RADIO);
		employeeNameRadio.setText("View by name");
		data = new GridData();
		data.horizontalSpan = 2;
		employeeNameRadio.setLayoutData(data);

		employeeNumRadio = new Button(employeeListComp, SWT.RADIO);
		employeeNumRadio.setText("View by ID");
		data = new GridData();
		data.horizontalSpan = 2;
		employeeNumRadio.setLayoutData(data);

		employeeNameRadio.setSelection(true);
		employeeNameRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				processor.setEmployeeView(TimeTrackerProc.VIEW_BY_NAME);
				refreshEmployeeData();
			}
		});

		employeeNumRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				processor.setEmployeeView(TimeTrackerProc.VIEW_BY_ID);
				refreshEmployeeData();
			}
		});
	}

	private void setupCurrentEmployee() {
		currentEmployeeComp = new Group(employeePane, SWT.NONE);
		currentEmployeeComp.setText("Current User:");

		//Form layout data for composite
		FormData formData = new FormData();
		formData.top = new FormAttachment(0, 5);
		formData.left = new FormAttachment(0, 5);
		//@Version ("0.2.2") - Changed from 50 to 60
		formData.bottom = new FormAttachment(0, 60);
		formData.right = new FormAttachment(33, -5);
		currentEmployeeComp.setLayoutData(formData);

		//Composite has a grid layout
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		currentEmployeeComp.setLayout(gridLayout);

		GridData data = new GridData();
		data.horizontalSpan = 2;

		currentEmployeeCombo =
			new Combo(
				currentEmployeeComp,
				SWT.DROP_DOWN | SWT.READ_ONLY);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 2;
		currentEmployeeCombo.setLayoutData(data);

		currentEmployeeCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(VIEW_MODE);
				setEmployeeMode(VIEW_MODE);

				//remove filter from employee list
				processor.setEmployeeFilter("");
				processor.setEmployeeView(TimeTrackerProc.VIEW_BY_NAME);
				employeeNameRadio.setSelection(true);
				employeeFilterText.setText("");

				//set active employee
				processor.setActiveEmployee(currentEmployeeCombo.getSelectionIndex());
				refreshEmployeeData();
			}
		});
	}

	private void setupProjectList() {
		projectListComp = new Group(employeePane, SWT.NONE);
		projectListComp.setText("Projects:");

		//Form layout data for composite
		FormData formData = new FormData();
		//@Version ("0.2.2") - Changed from 55 to 65
		formData.top = new FormAttachment(0, 65);
		formData.left = new FormAttachment(0, 5);
		formData.bottom = new FormAttachment(100, -5);
		formData.right = new FormAttachment(33, -5);
		projectListComp.setLayoutData(formData);

		//Composite has a grid layout
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		projectListComp.setLayout(gridLayout);

		GridData data = new GridData();
		data.horizontalSpan = 2;

		projectList =
			new List(
				projectListComp,
				SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 2;
		projectList.setLayoutData(data);

		projectList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(VIEW_MODE);
				setEmployeeMode(VIEW_MODE);
				processor.setActiveProject(projectList.getSelectionIndex());
				refreshTransactionData();
			}
		});


		//Create the filter text field and its listeners.
		Label filterLabel = new Label(projectListComp, SWT.NONE);
		filterLabel.setText("Filter:");
		projectFilterText = new Text(projectListComp, SWT.BORDER);
		GridData data1 = new GridData();
		//@Version ("0.2.2") - Changed from 26 to 34
		data1.widthHint = 34;
		filterLabel.setLayoutData(data1);
		GridData data2 = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		data2.widthHint = 48;
		projectFilterText.setLayoutData(data2);

		projectFilterText.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent arg0) {
				processor.setProjectFilter(projectFilterText.getText());
				refreshProjectData();
			}
		});

		projectFilterText.addListener(SWT.DefaultSelection, new Listener() {
			public void handleEvent(Event e) { //check for "enter" key
				processor.setProjectFilter(projectFilterText.getText());
				refreshProjectData();
			}
		});

		//Create radio buttons to switch viewing mode.
		projectNameRadio = new Button(projectListComp, SWT.RADIO);
		projectNameRadio.setText("View by name");
		data = new GridData();
		data.horizontalSpan = 2;
		projectNameRadio.setLayoutData(data);

		projectNumRadio = new Button(projectListComp, SWT.RADIO);
		projectNumRadio.setText("View by ID");
		data = new GridData();
		data.horizontalSpan = 2;
		projectNumRadio.setLayoutData(data);

		projectNameRadio.setSelection(true);

		projectNameRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				processor.setProjectView(TimeTrackerProc.VIEW_BY_NAME);
				refreshProjectData();
			}
		});

		projectNumRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				processor.setProjectView(TimeTrackerProc.VIEW_BY_ID);
				refreshProjectData();
			}
		});
	}

	private void setupProjectManagerProjectList() {
		Group pmProjectListComp = new Group(projectManagerPane, SWT.NONE);
		pmProjectListComp.setText("Projects:");

		//Form layout data for composite
		FormData formData2 = new FormData();
		formData2.top = new FormAttachment(0, 5);
		formData2.left = new FormAttachment(0, 5);
		formData2.bottom = new FormAttachment(100, -5);
		formData2.right = new FormAttachment(33, -5);
		pmProjectListComp.setLayoutData(formData2);


		//Composite has a grid layout
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		pmProjectListComp.setLayout(gridLayout);

		GridData data = new GridData();
		data.horizontalSpan = 2;

		pmProjectList =
			new List(
				pmProjectListComp,
				SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 2;
		pmProjectList.setLayoutData(data);

		pmProjectList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(VIEW_MODE);
				setEmployeeMode(VIEW_MODE);
				processor.setActiveAllProject(pmProjectList.getSelectionIndex());
				refreshPMProjectData();
				refreshAssignData();
			}
		});


		//Create the filter text field and its listeners.
		Label filterLabel = new Label(pmProjectListComp, SWT.NONE);
		filterLabel.setText("Filter:");
		pmProjectFilterText = new Text(pmProjectListComp, SWT.BORDER);
		GridData data1 = new GridData();
		//@Version ("0.2.2") - Changed from 26 to 34
		data1.widthHint = 34;
		filterLabel.setLayoutData(data1);
		GridData data2 = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		data2.widthHint = 48;
		pmProjectFilterText.setLayoutData(data2);

		pmProjectFilterText.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent arg0) {
				processor.setAllProjectFilter(pmProjectFilterText.getText());
				refreshPMProjectData();
			}
		});

		pmProjectFilterText.addListener(SWT.DefaultSelection, new Listener() {
			public void handleEvent(Event e) { //check for "enter" key
				processor.setAllProjectFilter(pmProjectFilterText.getText());
				refreshPMProjectData();
			}
		});

		//Create radio buttons to switch viewing mode.
		pmProjectNameRadio = new Button(pmProjectListComp, SWT.RADIO);
		pmProjectNameRadio.setText("View by name");
		data = new GridData();
		data.horizontalSpan = 2;
		pmProjectNameRadio.setLayoutData(data);

		pmProjectNumRadio = new Button(pmProjectListComp, SWT.RADIO);
		pmProjectNumRadio.setText("View by ID");
		data = new GridData();
		data.horizontalSpan = 2;
		pmProjectNumRadio.setLayoutData(data);

		pmProjectNameRadio.setSelection(true);

		pmProjectNameRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				processor.setAllProjectView(TimeTrackerProc.VIEW_BY_NAME);
				refreshPMProjectData();
			}
		});

		pmProjectNumRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				processor.setAllProjectView(TimeTrackerProc.VIEW_BY_ID);
				refreshPMProjectData();
			}
		});
	}

	private void setupEmployeePane() {
		employeeItem = new TabItem(folder, SWT.NONE);
		employeeItem.setText("&Employee Screen");
		employeePane = new Composite(folder, SWT.NONE);
		employeeItem.setControl(employeePane);

		//employeePane has a FormLayout
		employeePane.setLayout(new FormLayout());

		setupCurrentEmployee();

		setupProjectList();

		//employeeInfo has a GridLayout
		employeeInfo = new Group(employeePane, SWT.NONE);
		employeeInfo.setText("Employee Transactions:");
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		employeeInfo.setLayout(gridLayout);

		FormData formData = new FormData();
		formData.left = new FormAttachment(33, 5);
		formData.right = new FormAttachment(100, -5);
		formData.top = new FormAttachment(0, 5);
		formData.bottom = new FormAttachment(100, -5);
		employeeInfo.setLayoutData(formData);

		infoTable =
			new Table(
				employeeInfo,
				SWT.V_SCROLL | SWT.SINGLE | SWT.FULL_SELECTION | SWT.BORDER);

		GridData data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 4;
		data.heightHint = 160;
		infoTable.setLayoutData(data);

		infoTable.setLinesVisible(true);
		infoTable.setHeaderVisible(true);

		infoTable.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(VIEW_MODE);
				int index = infoTable.getSelectionIndex();

				int id = Integer.parseInt(infoTable.getItem(index).getText(0));

				Transaction current = processor.getTransactionByID(id);
				idText.setText("" + current.transactionID);
				amtText.setText("" + current.hours);
				dateText.setText(current.date);
				descText.setText(current.activity);
			}
		});

		ColumnSelectionListener sortListener = new ColumnSelectionListener();
		numberColumn = new TableColumn(infoTable, SWT.CENTER);
		numberColumn.setText("ID");
		numberColumn.setWidth(58);
		numberColumn.addSelectionListener(sortListener);

		amountColumn = new TableColumn(infoTable, SWT.RIGHT);
		amountColumn.setText("Hours");
		amountColumn.setWidth(80);
		amountColumn.addSelectionListener(sortListener);

		dateColumn = new TableColumn(infoTable, SWT.RIGHT);
		dateColumn.setText("Date");
		dateColumn.setWidth(80);
		dateColumn.addSelectionListener(sortListener);

		descColumn = new TableColumn(infoTable, SWT.LEFT);
		descColumn.setText("Activity");
		descColumn.setWidth(171);
		descColumn.addSelectionListener(sortListener);

		Label totalLabel = new Label(employeeInfo, SWT.NONE);
		totalLabel.setText("Total:");
		totalText = new Text(employeeInfo, SWT.BORDER | SWT.RIGHT);
		totalText.setEditable(false);

		data = new GridData();
		data.grabExcessHorizontalSpace = true;
		data.horizontalSpan = 3;
		totalText.setLayoutData(data);

		Label idLabel = new Label(employeeInfo, SWT.NONE);
		idLabel.setText("ID:");
		idText = new Text(employeeInfo, SWT.SINGLE | SWT.BORDER);
		idText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		idText.setEditable(false);

		Label descLabel = new Label(employeeInfo, SWT.NONE);
		descLabel.setText("Activity:");
		data = new GridData();
		data.verticalSpan = 3;
		descLabel.setLayoutData(data);
		descText = new Text(employeeInfo, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.WRAP);
		descText.setEditable(false);
		data = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_FILL);
		data.verticalSpan = 3;
		descText.setLayoutData(data);

		Label amtLabel = new Label(employeeInfo, SWT.NONE);
		amtLabel.setText("Hours:");
		amtText = new Text(employeeInfo, SWT.SINGLE | SWT.BORDER);
		amtText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		amtText.setEditable(false);

		Label dateLabel = new Label(employeeInfo, SWT.NONE);
		dateLabel.setText("Date:");
		dateText = new Text(employeeInfo, SWT.SINGLE | SWT.BORDER);
		final GridData gridData = new GridData();
		gridData.widthHint = 100;
		dateText.setLayoutData(gridData);
		dateText.setEditable(false);

		//buttonGrp has a FillLayout
		Composite buttonGrp = new Composite(employeeInfo, SWT.NONE);
		buttonGrp.setLayout(new FillLayout());
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 4;
		buttonGrp.setLayoutData(data);

		newTransButton = new Button(buttonGrp, SWT.PUSH);
		newTransButton.setText("&New");
		newTransButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(ADD_MODE);
			}
		});

		delTransButton = new Button(buttonGrp, SWT.PUSH);
		delTransButton.setText("&Delete");
		delTransButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				MessageBox msgBox =
					new MessageBox(
						shell,
						SWT.APPLICATION_MODAL | SWT.YES | SWT.NO);
				msgBox.setText("Confirm Deletion");
				msgBox.setMessage(
					"Are you certain you wish to delete the selected transaction?");
				int index = infoTable.getSelectionIndex();
				if (index >= 0 && msgBox.open() == SWT.YES) {
					processor.deleteTransaction(index);
					refreshEmployeeData();
				}
				else if (index < 0) {
					MessageBox msgBox2 =
						new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
					msgBox2.setText("Input Error");
					msgBox2.setMessage(
						"In order to delete a transaction, one must be selected.");
					msgBox2.open();
				}
			}
		});

		editTransButton = new Button(buttonGrp, SWT.PUSH);
		editTransButton.setText("&Edit");
		editTransButton.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent arg0) {
				editIndex = infoTable.getSelectionIndex();
				setTransMode(EDIT_MODE);
			}
		});

		//filler
		Label fill = new Label(buttonGrp, SWT.NONE);

		saveTransButton = new Button(buttonGrp, SWT.PUSH);
		saveTransButton.setText("&Save");
		saveTransButton.setEnabled(false);
		saveTransButton.addSelectionListener(new SaveTransButtonListener());

		cancelTransButton = new Button(buttonGrp, SWT.PUSH);
		cancelTransButton.setText("&Cancel");
		cancelTransButton.setEnabled(false);
		cancelTransButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setTransMode(VIEW_MODE);
			}
		});
	}

	private void setupAdministrationPane() {

		administrationItem = new TabItem(folder, SWT.NONE);
		administrationItem.setText("&Administration Screen");
		administrationPane = new Composite(folder, SWT.NONE);
		administrationItem.setControl(administrationPane);

		//administrationPane has a FormLayout
		administrationPane.setLayout(new FormLayout());

		setupEmployeeList();

		//administrationInfo has a FormLayout
		administrationInfo = new Group(administrationPane, SWT.NONE);
		administrationInfo.setText("Employee Information:");

		FormLayout administrationLayout = new FormLayout();
		administrationLayout.marginHeight = 10;
		administrationLayout.marginWidth = 10;
		administrationInfo.setLayout(administrationLayout);

		FormData formData = new FormData();
		formData.left = new FormAttachment(33, 5);
		formData.right = new FormAttachment(100, -5);
		formData.top = new FormAttachment(0, 5);
		formData.bottom = new FormAttachment(100, -5);
		administrationInfo.setLayoutData(formData);

		Label acctIDLabel = new Label(administrationInfo, SWT.NONE);
		acctIDLabel.setText("Employee ID:");
		acctIDText = new Text(administrationInfo, SWT.BORDER);
		acctIDText.setEditable(false);

		FormData formData2 = new FormData();
		formData2.left = new FormAttachment(0, 5);
		formData2.right = new FormAttachment(0, 135);
		formData2.top = new FormAttachment(0, 10);
		formData2.height = 15;
		acctIDLabel.setLayoutData(formData2);

		FormData formData2b = new FormData();
		formData2b.left = new FormAttachment(acctIDLabel, 10, SWT.DEFAULT);
		formData2b.right = new FormAttachment(100, -5);
		formData2b.top = new FormAttachment(acctIDLabel, -5, SWT.TOP);
		formData2b.height = 15;
		acctIDText.setLayoutData(formData2b);

		Label fnameLabel = new Label(administrationInfo, SWT.NONE);
		fnameLabel.setText("First name:");
		fnameText = new Text(administrationInfo, SWT.BORDER);
		fnameText.setEditable(false);

		FormData formData3 = new FormData();
		formData3.left = new FormAttachment(0, 5);
		formData3.right = new FormAttachment(0, 135);
		formData3.top = new FormAttachment(0, 35);
		formData3.height = 15;
		fnameLabel.setLayoutData(formData3);

		FormData formData3b = new FormData();
		formData3b.left = new FormAttachment(fnameLabel, 10, SWT.DEFAULT);
		formData3b.right = new FormAttachment(100, -5);
		formData3b.top = new FormAttachment(fnameLabel, -5, SWT.TOP);
		formData3b.height = 15;
		fnameText.setLayoutData(formData3b);

		Label snameLabel = new Label(administrationInfo, SWT.NONE);
		snameLabel.setText("Surname:");
		snameText = new Text(administrationInfo, SWT.BORDER);
		snameText.setEditable(false);

		FormData formData4 = new FormData();
		formData4.left = new FormAttachment(0, 5);
		formData4.right = new FormAttachment(0, 135);
		formData4.top = new FormAttachment(0, 60);
		formData4.height = 15;
		snameLabel.setLayoutData(formData4);

		FormData formData4b = new FormData();
		formData4b.left = new FormAttachment(snameLabel, 10, SWT.DEFAULT);
		formData4b.right = new FormAttachment(100, -5);
		formData4b.top = new FormAttachment(snameLabel, -5, SWT.TOP);
		formData4b.height = 15;
		snameText.setLayoutData(formData4b);

		Label dobLabel = new Label(administrationInfo, SWT.NONE);
		dobLabel.setText("Date of birth:");
		dobText = new Text(administrationInfo, SWT.BORDER);
		dobText.setEditable(false);

		FormData formData5 = new FormData();
		formData5.left = new FormAttachment(0, 5);
		formData5.right = new FormAttachment(0, 135);
		formData5.top = new FormAttachment(0, 85);
		formData5.height = 15;
		dobLabel.setLayoutData(formData5);

		FormData formData5b = new FormData();
		formData5b.left = new FormAttachment(dobLabel, 10, SWT.DEFAULT);
		formData5b.right = new FormAttachment(100, -5);
		formData5b.top = new FormAttachment(dobLabel, -5, SWT.TOP);
		formData5b.height = 15;
		dobText.setLayoutData(formData5b);

		Label phoneLabel = new Label(administrationInfo, SWT.NONE);
		phoneLabel.setText("Phone number:");
		phoneText = new Text(administrationInfo, SWT.BORDER);
		phoneText.setEditable(false);

		FormData formData6 = new FormData();
		formData6.left = new FormAttachment(0, 5);
		formData6.right = new FormAttachment(0, 135);
		formData6.top = new FormAttachment(0, 110);
		formData6.height = 15;
		phoneLabel.setLayoutData(formData6);

		FormData formData6b = new FormData();
		formData6b.left = new FormAttachment(phoneLabel, 10, SWT.DEFAULT);
		formData6b.right = new FormAttachment(100, -5);
		formData6b.top = new FormAttachment(phoneLabel, -5, SWT.TOP);
		formData6b.height = 15;
		phoneText.setLayoutData(formData6b);

		Label emailLabel = new Label(administrationInfo, SWT.NONE);
		emailLabel.setText("Email address:");
		emailText = new Text(administrationInfo, SWT.BORDER);
		emailText.setEditable(false);

		FormData formData7 = new FormData();
		formData7.left = new FormAttachment(0, 5);
		formData7.right = new FormAttachment(0, 135);
		formData7.top = new FormAttachment(0, 135);
		formData7.height = 15;
		emailLabel.setLayoutData(formData7);

		FormData formData7b = new FormData();
		formData7b.left = new FormAttachment(emailLabel, 10, SWT.DEFAULT);
		formData7b.right = new FormAttachment(100, -5);
		formData7b.top = new FormAttachment(emailLabel, -5, SWT.TOP);
		formData7b.height = 15;
		emailText.setLayoutData(formData7b);

		Label addressLabel = new Label(administrationInfo, SWT.NONE);
		addressLabel.setText("Address:");
		addressText =
			new Text(
				administrationInfo,
				SWT.WRAP | SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);
		addressText.setEditable(false);

		FormData formData8 = new FormData();
		formData8.left = new FormAttachment(0, 8);
		formData8.right = new FormAttachment(0, 135);
		formData8.top = new FormAttachment(0, 180);
		formData8.height = 15;
		addressLabel.setLayoutData(formData8);

		FormData formData8b = new FormData();
		formData8b.bottom = new FormAttachment(0, 230);
		formData8b.left = new FormAttachment(addressLabel, 10, SWT.DEFAULT);
		formData8b.right = new FormAttachment(100, -5);
		formData8b.top = new FormAttachment(emailLabel, 5, SWT.BOTTOM);
		formData8b.height = 15;
		addressText.setLayoutData(formData8b);

		//@Version ("0.2.2") - Changed from SWT.NONE to SWT.WRAP
		Label miscInfoLabel = new Label(administrationInfo, SWT.WRAP);
		miscInfoLabel.setText("Miscellaneous information:");
		miscInfoText =
			new Text(
				administrationInfo,
				SWT.WRAP | SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);
		miscInfoText.setEditable(false);

		FormData formData9 = new FormData();
		formData9.left = new FormAttachment(0, 8);
		formData9.right = new FormAttachment(0, 135);
		formData9.top = new FormAttachment(0, 260);
		//@Version ("0.2.2") - Changed from 15 to 30 for SWT.WRAP
		formData9.height = 30;
		miscInfoLabel.setLayoutData(formData9);

		FormData formData9b = new FormData();
		formData9b.bottom = new FormAttachment(0, 310);
		formData9b.left = new FormAttachment(miscInfoLabel, 10, SWT.DEFAULT);
		formData9b.right = new FormAttachment(100, -5);
		formData9b.top = new FormAttachment(addressText, 5, SWT.BOTTOM);
		formData9b.height = 15;
		miscInfoText.setLayoutData(formData9b);

		//buttonGrp has a FillLayout
		Composite buttonGrp = new Composite(administrationInfo, SWT.NONE);
		buttonGrp.setLayout(new FillLayout());

		FormData data3 = new FormData();
		data3.bottom = new FormAttachment(100, 0);
		data3.left = new FormAttachment(0, 0);
		data3.right = new FormAttachment(100, 0);
		data3.top = new FormAttachment(100, -25);
		buttonGrp.setLayoutData(data3);

		newEmployeeButton = new Button(buttonGrp, SWT.PUSH);
		newEmployeeButton.setText("&New");
		newEmployeeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setEmployeeMode(ADD_MODE);
			}
		});

		delEmployeeButton = new Button(buttonGrp, SWT.PUSH);
		delEmployeeButton.setText("&Delete");
		delEmployeeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				Employee c = processor.getActiveEmployee();
				
				//Can't delete an employee if it is the manager of a project.
				for (int i = 0; i < processor.getNumAllProjects(); i++) {
					if (processor.getAllProjectByIndex(i).managerID == c.employeeID) {
						MessageBox msgBox =
								new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
							msgBox.setText("Error");
							msgBox.setMessage(
								"Cannot delete selected employee because selected "
								+ "employee is the manager of a project.");
							msgBox.open();
							return;
					}
				}
				
				MessageBox msgBox =
					new MessageBox(
						shell,
						SWT.APPLICATION_MODAL | SWT.YES | SWT.NO);
				msgBox.setText("Confirm Deletion");
				msgBox.setMessage(
					"Are you certain you wish to delete the selected employee?\n"
						+ "(All related transactions will also  be deleted.)");

				if (msgBox.open() == SWT.YES) {
					processor.deleteEmployee(employeeList.getSelectionIndex());
					refreshEmployeeData();
					setEmployeeMode(VIEW_MODE);
				}
			}
		});

		editEmployeeButton = new Button(buttonGrp, SWT.PUSH);
		editEmployeeButton.setText("&Edit");
		editEmployeeButton.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent arg0) {
				setEmployeeMode(EDIT_MODE);
			}
		});

		//filler
		Label fill = new Label(buttonGrp, SWT.NONE);

		saveEmployeeButton = new Button(buttonGrp, SWT.PUSH);
		saveEmployeeButton.setText("&Save");
		saveEmployeeButton.setEnabled(false);
		saveEmployeeButton.addSelectionListener(new SaveEmployeeButtonListener());

		cancelEmployeeButton = new Button(buttonGrp, SWT.PUSH);
		cancelEmployeeButton.setText("&Cancel");
		cancelEmployeeButton.setEnabled(false);
		cancelEmployeeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setEmployeeMode(VIEW_MODE);
				refreshEmployeeData();
			}
		});

		//filler2
		Label fill2 = new Label(buttonGrp, SWT.NONE);

		printTransButton = new Button(buttonGrp, SWT.PUSH);
		printTransButton.setText("&Print");
		printTransButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				//Cancel any other actions currently in progress
				setTransMode(VIEW_MODE);

				MessageBox msgBox =
					new MessageBox(
						shell,
						SWT.APPLICATION_MODAL | SWT.YES | SWT.NO);
				msgBox.setText("Print Report");
				msgBox.setMessage(
					"Are you sure you would like to print the\naccount statements for the selected employee?\n");

				if (msgBox.open() != SWT.YES)
					return;

				//Create styled text widget and print it on the default printer
				Font f = new Font(display, "Courier New", 12, 0);
				printText.setFont(f);
				printText.setWordWrap(true);
				Printer p = new Printer();
				int width = p.getBounds().width;

				//Create text string to be printed
				StringBuffer report = new StringBuffer("Employee Report - ");
				Employee c = processor.getActiveEmployee();
				report.append(c.firstName + " " + c.lastName + "\n");
				int len = report.length();
				for (int i = 0; i < len; i++)
					report.append('-');
				report.append("\n");
				report.append("Account ID: " + c.employeeID + "\n");
				report.append("First Name: " + c.firstName + "\n");
				report.append("Last Name: " + c.lastName + "\n");
				report.append("Date of birth: " + c.dob + "\n");
				report.append("Phone Number: " + c.phone + "\n");
				report.append("Email: " + c.email + "\n");
				report.append("Address:\n" + c.address + "\n");
				report.append(
					"Miscellaneous information:\n" + c.miscInfo + "\n");

				report.append("\nTransactions\n------------\n\n");
				report.append("ID   Project Hours     Date      Activity\n");
				report.append("--   ------- -----     ----      --------\n");

				//This depends on all characters having identical widths
				//for the table to be laid out approximately correct.
				//Also, dollar values should be laid out with the decimal
				//aligned rather than the leftmost character.
				Transaction[] transList = processor.getTransactionsByEmployee(c.employeeID);
				int totalHours = 0;
				
				for (int i = 0; i < transList.length; i++) {
					StringBuffer trans = new StringBuffer("" + transList[i].transactionID);
					while (trans.length() < 7)
						trans.append(' ');

					trans.append(transList[i].projectID);
					while (trans.length() < 14)
						trans.append(' ');
					
					trans.append(transList[i].hours);
					totalHours += transList[i].hours;
					while (trans.length() < 20)
						trans.append(' ');

					trans.append(transList[i].date + "   ");

					int lineBreak = transList[i].activity.indexOf("\n");
					String descLine =
						lineBreak < 0
							? transList[i].activity
							: transList[i].activity.substring(0, lineBreak);
					trans.append(descLine);

					//truncate trans to fit on one line...

					trans.append('\n');

					report.append(trans.toString());
				}

				report.append('\n');
				report.append("Total hours: " + totalHours);

				printText.setText(report.toString());
				printText.setSize(width, 10000);
				printText.print();
				f.dispose();
			}
		});
	}

	private void setupProjectManagerPane() {
		projectManagerItem = new TabItem(folder, SWT.NONE);
		projectManagerItem.setText("&Project Manager Screen");
		projectManagerPane = new Composite(folder, SWT.NONE);
		projectManagerItem.setControl(projectManagerPane);

		setupProjectManagerProjectList();

		//projectManagerPane has a FormLayout
		projectManagerPane.setLayout(new FormLayout());

		//Set up the Project Info group
		Group projectManagerInfo = new Group(projectManagerPane, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		projectManagerInfo.setLayout(gridLayout);
		projectManagerInfo.setText("Project Information:");

		FormData formData3 = new FormData();
		formData3.left = new FormAttachment(33, 5);
		formData3.right = new FormAttachment(100, -5);
		formData3.top = new FormAttachment(0, 5);
		//@Version ("0.2.2") - Changed from 130 to 150
		formData3.bottom = new FormAttachment(0, 150);
		projectManagerInfo.setLayoutData(formData3);

		Label idLabel = new Label(projectManagerInfo, SWT.NONE);
		idLabel.setText("Project ID:");

		projectIDText = new Text(projectManagerInfo, SWT.BORDER);
		projectIDText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		projectIDText.setEditable(false);

		Label nameLabel = new Label(projectManagerInfo, SWT.NONE);
		nameLabel.setText("Project Name:");

		projectNameText = new Text(projectManagerInfo, SWT.BORDER);
		projectNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		projectNameText.setEditable(false);

		Label managerLabel = new Label(projectManagerInfo, SWT.NONE);
		managerLabel.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
		managerLabel.setText("Project Manager:");

		pmCombo = new Combo(projectManagerInfo, SWT.READ_ONLY);
		pmCombo.setEnabled(false);
		pmCombo.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_BEGINNING));

		//buttonGrp has a FillLayout
		Composite buttonGrp = new Composite(projectManagerInfo, SWT.NONE);
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData.horizontalSpan = 2;
		buttonGrp.setLayoutData(gridData);
		buttonGrp.setLayout(new FillLayout());

		newProjectButton = new Button(buttonGrp, SWT.PUSH);
		newProjectButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setProjectMode(ADD_MODE);
			}
		});
		newProjectButton.setText("&New");

		delProjectButton = new Button(buttonGrp, SWT.PUSH);
		delProjectButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//No project is selected
				if (pmProjectList.getSelectionIndex() == -1) {
					MessageBox msgBox =
						new MessageBox(
							shell,
							SWT.APPLICATION_MODAL | SWT.OK);
					msgBox.setText("No Selection");
					msgBox.setMessage("Please select a project to delete.");
					msgBox.open();
					
					return;
				}
				
				MessageBox msgBox =
					new MessageBox(
						shell,
						SWT.APPLICATION_MODAL | SWT.YES | SWT.NO);
				msgBox.setText("Confirm Deletion");
				msgBox.setMessage(
					"Are you certain you wish to delete the selected project?\n"
						+ "(All related transactions will also  be deleted.)");

				if (msgBox.open() == SWT.YES) {
					processor.deleteProject(pmProjectList.getSelectionIndex());
					refreshEmployeeData();
					refreshPMProjectData();
				}
			}
		});
		delProjectButton.setText("&Delete");

		editProjectButton = new Button(buttonGrp, SWT.PUSH);
		editProjectButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setProjectMode(EDIT_MODE);
			}
		});
		editProjectButton.setText("&Edit");
		
		//filler
		Label fill = new Label(buttonGrp, SWT.NONE);

		saveProjectButton = new Button(buttonGrp, SWT.PUSH);
		saveProjectButton.setEnabled(false);
		saveProjectButton.setText("&Save");
		saveProjectButton.addSelectionListener(new SaveProjectButtonListener());

		cancelProjectButton = new Button(buttonGrp, SWT.PUSH);
		cancelProjectButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setProjectMode(VIEW_MODE);
			}
		});
		cancelProjectButton.setEnabled(false);
		cancelProjectButton.setText("&Cancel");

		//Set up the assign employee to project group
		Group employeeProjectGroup = new Group(projectManagerPane, SWT.NONE);
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.makeColumnsEqualWidth = true;
		gridLayout_1.numColumns = 2;
		employeeProjectGroup.setLayout(gridLayout_1);
		employeeProjectGroup.setText("Employee Assignment:");
		
		FormData formData = new FormData();
		formData.bottom = new FormAttachment(100, -5);
		formData.right = new FormAttachment(100, -5);
		//@Version ("0.2.2") - Changed from 135 to 155
		formData.top = new FormAttachment(0, 155);
		formData.left = new FormAttachment(33, 5);
		employeeProjectGroup.setLayoutData(formData);

		Label notInProjectLabel = new Label(employeeProjectGroup, SWT.NONE);
		notInProjectLabel.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL));
		notInProjectLabel.setText("Employees not in Project:");

		Label inProjectLabel = new Label(employeeProjectGroup, SWT.NONE);
		inProjectLabel.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL));
		inProjectLabel.setText("Employees in Project:");

		notInProjectList = new List(employeeProjectGroup, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		notInProjectList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				inProjectList.deselectAll();
			}
		});
		notInProjectList.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_VERTICAL));

		inProjectList = new List(employeeProjectGroup, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		inProjectList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				notInProjectList.deselectAll();
			}
		});
		inProjectList.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_VERTICAL));

		pmEmployeeNameRadio = new Button(employeeProjectGroup, SWT.RADIO);
		pmEmployeeNameRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				int inIndex = inProjectList.getSelectionIndex();
				int notInIndex = notInProjectList.getSelectionIndex();
				int inID = -1;
				int notInID = -1;
				
				//Get the selected employee
				if (inIndex != -1)
					inID = processor.getEmployeeInProjectByIndex(inIndex).employeeID;
				if (notInIndex != -1)
					notInID = processor.getEmployeeNotInProjectByIndex(notInIndex).employeeID;
				
				processor.setInProjectView(TimeTrackerProc.VIEW_BY_NAME);
				refreshAssignData();
				
				//Select previously selected employee
				notInProjectList.deselectAll();
				inProjectList.select(processor.getInProjectIndex(inID));
				notInProjectList.select(processor.getNotInProjectIndex(notInID));
			}
		});
		pmEmployeeNameRadio.setSelection(true);
		pmEmployeeNameRadio.setText("View by name");

		pmEmployeeNumRadio = new Button(employeeProjectGroup, SWT.RADIO);
		pmEmployeeNumRadio.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				int inIndex = inProjectList.getSelectionIndex();
				int notInIndex = notInProjectList.getSelectionIndex();
				int inID = -1;
				int notInID = -1;
				
				//Get the selected employee
				if (inIndex != -1)
					inID = processor.getEmployeeInProjectByIndex(inIndex).employeeID;
				if (notInIndex != -1)
					notInID = processor.getEmployeeNotInProjectByIndex(notInIndex).employeeID;
												
				processor.setInProjectView(TimeTrackerProc.VIEW_BY_ID);
				refreshAssignData();
				
				//Select the previously selected employee
				notInProjectList.deselectAll();
				inProjectList.select(processor.getInProjectIndex(inID));
				notInProjectList.select(processor.getNotInProjectIndex(notInID));
			}
		});
		pmEmployeeNumRadio.setText("View by ID");

		//Set up the Assign and Remove buttons
		Composite buttonGrp2 = new Composite(employeeProjectGroup, SWT.NONE);
		final GridData gridData_1 = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gridData_1.horizontalSpan = 2;
		buttonGrp2.setLayoutData(gridData_1);
		buttonGrp2.setLayout(new FillLayout());

		assignButton = new Button(buttonGrp2, SWT.PUSH);
		assignButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//If an employee is not selected in the notInProject list
				if (notInProjectList.getSelectionIndex() == -1) {
					MessageBox msgBox =
							new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
						msgBox.setText("Input Error");
						msgBox.setMessage(
							"An employee not currently in the project must be selected.");
						msgBox.open();
						return;
				}
				
				//If a project is not selected
				if (pmProjectList.getSelectionIndex() == -1) {
					MessageBox msgBox =
							new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
						msgBox.setText("Input Error");
						msgBox.setMessage(
							"A project must be selected.");
						msgBox.open();
						return;
				}
				
				Employee employee = 
					processor.getEmployeeNotInProjectByIndex(notInProjectList.getSelectionIndex());
				Project project = processor.getActiveAllProject();
				
				processor.addEmployeeProject(employee, project);

				refreshAssignData();
				refreshEmployeeData();

				notInProjectList.deselectAll();
				inProjectList.select(processor.getInProjectIndex(employee.employeeID));
			}
		});
		assignButton.setText("&Assign Employee to Project");

		unassignButton = new Button(buttonGrp2, SWT.PUSH);
		unassignButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//If an employee is not selected in the inProject list
				if (inProjectList.getSelectionIndex() == -1) {
					MessageBox msgBox =
							new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
						msgBox.setText("Error");
						msgBox.setMessage(
							"An employee currently in the project must be selected.");
						msgBox.open();
						return;
				}
				
				//If a project is not selected
				if (pmProjectList.getSelectionIndex() == -1) {
					MessageBox msgBox =
							new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
						msgBox.setText("Error");
						msgBox.setMessage(
							"A project must be selected.");
						msgBox.open();
						return;
				}
				
				Employee employee = 
					processor.getEmployeeInProjectByIndex(inProjectList.getSelectionIndex());	
				int managerID = processor.getEmployeeByIndex(pmCombo.getSelectionIndex()).employeeID;
				
				//If the employee to be unassigned is the project manager
				if (employee.employeeID == managerID) {
					MessageBox msgBox =
							new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
						msgBox.setText("Error");
						msgBox.setMessage(
							"Cannot unassign employee when employee is project manager.");
						msgBox.open();
						return;
				}
				
				Project project = processor.getActiveAllProject();
				
				processor.deleteEmployeeProject(employee, project);

				refreshAssignData();
				refreshEmployeeData();
				
				inProjectList.deselectAll();
				notInProjectList.select(processor.getNotInProjectIndex(employee.employeeID));
			}
		});
		unassignButton.setText("&Remove Employee from Project");
	}

	private void setupFileMenu() {
		//create the menu bar
		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menu);

		//add the File option to it
		MenuItem file = new MenuItem(menu, SWT.CASCADE);
		file.setText("File");

		//create a menu for the File option
		Menu filemenu = new Menu(shell, SWT.DROP_DOWN);
		file.setMenu(filemenu);

		//add MenuItems to the File menu
		MenuItem aboutItem = new MenuItem(filemenu, SWT.PUSH);
		aboutItem.setText("About...");

		MenuItem quitItem = new MenuItem(filemenu, SWT.PUSH);
		quitItem.setText("Quit");

		//add a listener for the action
		quitItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				shell.dispose();
			}
		});

		aboutItem.addListener(SWT.Selection, new Listener() {
			private Image aboutImage;
			private Shell aboutShell;

			public void handleEvent(Event e) {

				aboutShell = new Shell(shell, SWT.DIALOG_TRIM);
				aboutShell.setSize(200, 250);
				aboutShell.setText("About...");

				//load an Image
				aboutImage = new Image(display, "splash.jpg");

				aboutShell.addPaintListener(new PaintListener() {
					public void paintControl(PaintEvent e) {
						//Get the graphics context
						GC gc = new GC(aboutShell);
						//Write a string to the shell
						gc.drawString(
							"TimeTracker Application " + VERSION,
							30,
							10);

						//draw a blue rectangle
						Color blue = display.getSystemColor(SWT.COLOR_BLUE);
						gc.setForeground(blue);
						gc.drawRectangle(30, 30, 135, 165);

						//draw an image
						gc.drawImage(aboutImage, 48, 48);
					}
				});

				aboutShell.addDisposeListener(new DisposeListener() {
					public void widgetDisposed(DisposeEvent e) {
						//when the shell is closed, dispose of the Image
						aboutImage.dispose();
						shell.setEnabled(true);
					}
				});

				shell.setEnabled(false);
				aboutShell.open();
			}
		});
	}

	class SaveTransButtonListener extends SelectionAdapter {

		//respond to presses of the Save button on the transaction page
		public void widgetSelected(SelectionEvent arg0) {
			int transID;
			int amt;

			try {
				transID = Integer.parseInt(idText.getText());
				amt = Integer.parseInt(amtText.getText());
			}
			catch (NumberFormatException e) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"The transaction ID and hours entered must be valid integers.");
				msgBox.open();
				return;
			}

			boolean success = false;
			if (transactionMode == ADD_MODE) {
				success =
					processor.addTransaction(
						transID,
						processor.getActiveProject().projectID,
						amt,
						dateText.getText(),
						descText.getText());
			}
			else if (transactionMode == EDIT_MODE) {
				success =
					processor.updateTransaction(
						editIndex,
						transID,
						processor.getActiveProject().projectID,
						amt,
						dateText.getText(),
						descText.getText());
			}
			if (!success) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"The Transaction ID entered must be a valid integer unique among transactions for this employee.");
				msgBox.open();
				return;
			}

			refreshTransactionData();
			setTransMode(VIEW_MODE);
		}
	}

	class SaveEmployeeButtonListener extends SelectionAdapter {

		//respond to presses of the Save button on the administration page
		public void widgetSelected(SelectionEvent arg0) {
			int employeeID;

			try {
				employeeID = Integer.parseInt(acctIDText.getText());
			}
			catch (NumberFormatException e) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"The employee ID entered must be a valid integer unique among all employees.");
				msgBox.open();
				return;
			}

			boolean success = false;
			if (employeeMode == ADD_MODE) {
				success =
					processor.addEmployee(
						employeeID,
						fnameText.getText(),
						snameText.getText(),
						phoneText.getText(),
						emailText.getText(),
						dobText.getText(),
						addressText.getText(),
						miscInfoText.getText());
			}
			else if (employeeMode == EDIT_MODE) {
				success =
					processor.updateEmployee(
						employeeID,
						fnameText.getText(),
						snameText.getText(),
						phoneText.getText(),
						emailText.getText(),
						dobText.getText(),
						addressText.getText(),
						miscInfoText.getText());
			}

			if (!success) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"The Employee ID entered must be a valid integer unique among all employees.");
				msgBox.open();
				return;
			}

			refreshEmployeeData();
			refreshPMProjectData();
			refreshAssignData();
			setEmployeeMode(VIEW_MODE);
		}
	}

	class SaveProjectButtonListener extends SelectionAdapter {

		//respond to presses of the Save button on the project manager page
		public void widgetSelected(SelectionEvent arg0) {
			int projectID;
			int managerID;

			try {
				projectID = Integer.parseInt(projectIDText.getText());
			}
			catch (NumberFormatException e) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"The project ID entered must be a valid integer unique among all projects.");
				msgBox.open();
				return;
			}

			if (pmCombo.getSelectionIndex() == -1) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"No employees are available to be project manager.  Please create an employee first.");
				msgBox.open();
				setProjectMode(VIEW_MODE);
				return;
			}

			managerID = processor.getEmployeeByIndex(pmCombo.getSelectionIndex()).employeeID;

			boolean success = false;
			if (projectMode == ADD_MODE) {
				success =
					processor.addProject(
						projectID,
						projectNameText.getText(),
						managerID);
			}
			else if (projectMode == EDIT_MODE) {
				success =
					processor.updateProject(
						projectID,
						projectNameText.getText(),
						managerID);
			}

			if (!success) {
				MessageBox msgBox =
					new MessageBox(shell, SWT.APPLICATION_MODAL | SWT.OK);
				msgBox.setText("Input Error");
				msgBox.setMessage(
					"The project ID entered must be a valid integer unique among all projects.");
				msgBox.open();
				return;
			}

			refreshEmployeeData();
			refreshPMProjectData();
			refreshAssignData();
			setProjectMode(VIEW_MODE);
		}
	}

	private void sortTransactions(int criteria) {

		TableItem[] items = infoTable.getItems();

		Arrays.sort(items, new TransactionComparator(criteria));
		for (int i = 0; i < items.length; i++) {
			TableItem t = new TableItem(infoTable, SWT.NONE);
			String[] data = new String[infoTable.getColumnCount()];

			for (int j = 0; j < data.length; j++)
				data[j] = items[i].getText(j);

			items[i].dispose();
			t.setText(data);
		}
	}

	class TransactionComparator implements java.util.Comparator {
		private int criteria;

		TransactionComparator(int c) {
			criteria = c;
		}

		public int compare(Object arg0, Object arg1) {
			boolean numeric = criteria == SORT_BY_AMT || criteria == SORT_BY_ID;
			TableItem first = (TableItem) arg0;
			TableItem second = (TableItem) arg1;

			if (criteria == SORT_BY_AMT) {
				Float f = new Float(first.getText(criteria));
				Float s = new Float(second.getText(criteria));
				return f.compareTo(s) < 0 ? 1 : -1;
			}
			else if (criteria == SORT_BY_ID) {
				int f = Integer.parseInt(first.getText(criteria));
				int s = Integer.parseInt(second.getText(criteria));
				return f - s;
			}
			else if (criteria == SORT_BY_DATE) {
				return first.getText(criteria).compareTo(
					second.getText(criteria))
					< 0
					? 1
					: -1;
			}
			else if (criteria == SORT_BY_DESC) {
				return first.getText(criteria).compareTo(
					second.getText(criteria))
					< 0
					? -1
					: 1;
			}
			else
				throw new RuntimeException("Unexpected Internal Error.");
		}
	}

	class ColumnSelectionListener extends SelectionAdapter {

		public void widgetSelected(SelectionEvent arg0) {
			int criteria = -1;

			if (arg0.getSource() == numberColumn) {
				criteria = SORT_BY_ID;
			}
			else if (arg0.getSource() == dateColumn) {
				criteria = SORT_BY_DATE;
			}
			else if (arg0.getSource() == descColumn) {
				criteria = SORT_BY_DESC;
			}
			else if (arg0.getSource() == amountColumn) {
				criteria = SORT_BY_AMT;
			}
			else {
				throw new RuntimeException("Unexpected Internal Error.");
			}

			sortTransactions(criteria);
		}
	}
}
